ProbBin.FCS <- function(controldata, stimuldata, N=100, varname="",
                        PBspec=c("by.control", "combined"),
                        MY.DEBUG=TRUE,...) {

#################################################
  # Objective:
  ###############################################
  # Make a ProbBin.FCS object
  ###############################################
  # Input variables:
  ###############################################
  # Name       Description
  #############################
  # controldata      unstimulated sample data
  #                    (of 1 variable) for a given individual
  # stimuldata      the stimulated sample (1 variable) for a given individual
  # N          The number of cells/observations per a bin
  
  # PBspec     The type of probability binning either:
  #               "by.control" in which the breaks for the bins are
  #               based on the unstimulated having N observations in each bin
  #               "combined" in which the breaks for the bins are
  #               based on the combined dataset
  #               (stimulated and unstimulated) having
  #               N observations in each bin

  ####################################################################
  # Output Results:
  #######################################################################
  # A ProbBin.FCS object is returned which has the following list components:
  #
  # Name                Description
  #
  # st.hist    a histogram object of the stimulated data
  # unst.hist  a histogram object of the unstimulated data
  # PB         either "by.control" or "combined"
  # N.in.bin   number in each bin for the combined or
  #              the control data (depending on PBspec & PB)
  # 
  # 
  ######################################################################
  
 

  PBspec <- match.arg(PBspec)
  if (PBspec=="by.control"){
    breaks<-breakpoints.ProbBin(controldata, N)
    breaks.st <- NULL
    
    ## check the max of st.var
    if (max(controldata) > max(stimuldata)){
      ## remove the breaks that are greater than the max value of st.var
      breaks.st.mid <- unlist(sapply(breaks,
                                    function(x){
                                      if (x < max(stimuldata)){
                                        x
                                      }
                                      }))
  
      breaks.st <- c(breaks.st.mid, max(stimuldata))
      
    }

    if (max(stimuldata) > max(controldata)){
      ## add on max value of st.var to the breaks that are less than it
      ## essentially the last interval will be max(breaks) to max(st.var)
     
        breaks.st <- c(breaks[-length(breaks)], max(stimuldata))
     
    }

    if (min(stimuldata) > min(controldata)){
      if (is.null(breaks.st)){
        ## remove
        breaks.st.mid <- unlist(sapply(breaks,
                                       function(x){
                                         if (x > min(stimuldata)){
                                           x
                                         }
                                       }))
        
        breaks.st <- c(min(stimuldata)-1, breaks.st.mid)
      } else {
         breaks.st.mid <- unlist(sapply(breaks.st,
                                       function(x){
                                         if (x > min(stimuldata)){
                                           x
                                         }
                                       }))
        
        breaks.st <- c(min(stimuldata)-1, breaks.st.mid)
        
      }
    }

    if (min(stimuldata) < min(controldata)){
      if (is.null(breaks.st)){
      ## add the minimum-1 onto the breaks.st
        breaks.st <- c(min(stimuldata)-1, breaks[-1])
      } else {
        breaks.st <- c(min(stimuldata)-1, breaks.st[-1])

      }

    }
  } else if (PBspec=="combined"){
    comb.var <- c(controldata, stimuldata)
    breaks<-breakpoints.ProbBin(comb.var, N)
    breaks.st <- breaks
  }

  
  if(MY.DEBUG==TRUE){
    print(breaks)
    print(c(min(controldata), max(controldata)))
  }
  
  unst.hist<-hist(controldata,  br=breaks,
                  ..., plot=FALSE)

 
  ##check for the max of the stimuldata
  if (MY.DEBUG==TRUE) {
    print("Length of the control data:")
    print(length(controldata))
    print("Length of the stimulated data:")
    print(length(stimuldata))
  }
  st.hist<-hist(stimuldata, br=breaks.st,
                ..., plot=FALSE)

  diff.bins <- length(st.hist$counts) - length(unst.hist$counts)

  if (diff.bins<0) {  # count.unst has more bins
    st.hist$counts<- c(st.hist$counts, rep(0,abs(diff.bins)))
    st.hist$breaks <- c(st.hist$breaks,( max(stimuldata)+ 1:abs(diff.bins)))
  }else if (diff.bins>0) { #count st has more bins, so put all obs in last bin 
    st.hist$counts <- c(st.hist$counts[1:(length(unst.hist$counts)-1)],
                        sum(st.hist$counts[length(unst.hist$counts):length(st.hist$counts)])) 
    st.hist$breaks <- st.hist$breaks[1:length(unst.hist$breaks)]
  }
  
###  need to account for zero bins for by.control   
  structure(list(unst.hist=unst.hist,
                 st.hist=st.hist, PB=PBspec, N.in.bin=as.numeric(N),
                 varname=varname),
            class="ProbBin.FCS")
}
