summary.ProbBin.FCS <- function(object, verbose=FALSE, ...){

  # object   ProbBin.FCS object
 
  
  ############################################################
  ##  OUTPUT
  ############################################################
  ## 1. table summary with statistics and pearson's chi-sq test
  ##    (pg. 144, Keith A. Baggerly "Probability Binning and Test
  ##    Agreement between Multivariate Immunofluorescence Histograms:
  ##    Extending the Chi-Squared test" Cytometry 45: 141-150 (2001)
  ##

  if(is(object, "ProbBin.FCS")==FALSE){
    stop("Input is not a ProbBin.FCS object")
  }

  AnalyType <- object$PB
  unst.hist <- object$unst.hist
  st.hist <- object$st.hist
  N <- object$N.in.bin
    ## PB metric
  c.i <- unst.hist$counts
  s.i <- st.hist$counts
  n.c <- sum(c.i)
  n.s <- sum(s.i)
  
  n <- min(n.c, n.s)
  B <- length(c.i)
  
  c.i.1 <- c.i/n.c
  s.i.1 <- s.i/n.s
  
  cat("\n")
  cat("          ",
      "Test of distribution difference: Probability Binning & PB metric",
      "\n")
  
  cat("\n")
  cat("\n")
  cat("Null Hypothesis: Unstimulated/Control Data Histogram/Bins are the", "\n")
  cat("   statistically the same as the Stimulated Data Histogram/Bins;", "\n")
  cat("   both samples are from the same distribution",
      "\n")
  cat("Alternative Hypothesis: Unstimulated/Control Data Histogram/Bins", "\n")
  cat("   are significantly different from the Stimulated Data Histogram/Bins;", "\n")
  cat("   the stimulated and unstimulated samples are from different distributions", "\n")
  cat("\n")
  word1 <- ifelse(AnalyType=="by.control", "control", "combined (control & stimulated)")
  
  cat(paste("Bins obtained from Probability binning with", N, "\n"))
  cat(paste("   in each bin in the", word1, "dataset", sep=" "), "\n")
  cat(paste("Note: The counts in the first bin may be greater than ",  N, "\n"))
  cat(paste("   because of abundance of zero data.", "\n"))
  out.mst <- cbind(c.i, s.i)
  colnames(out.mst) <- c("Unstimulated/Control", "Stimulated/Case")
  if (!verbose){
    cat("The counts in the bins are not shown because there are too many bins.", "\n")
  } else {
    print(out.mst)
  }
  ##  cat("   Unstimulated/Control:", paste(c.i, collapse=","), "\n")
  cat("        Number of Control:", n.c, "\n")
  ##  cat("   Stimulated:", paste(s.i, collapse=","), "\n")
  cat("        Number of Stimulated:", n.s, "\n")
  
  ##  Roederder's unadjusted chi.sq/PBmetric
  chi.sq.unadj <- sum(apply(data.frame(c.i.1, s.i.1), 1,
                            function(x){ ((x[1]-x[2])^2) / (x[1]+x[2]) } ))
  
  mean.unadj <- B/n
  st.dev.unadj <- sqrt(B) / n
  
  ## T.chi.unadj=max(0,(PBmetric-mean(PBmetric)) / SD(PBmetric)) is approximately standard normal (CLT)
  T.chi.unadj <- max(c(0,(chi.sq.unadj-mean.unadj) / st.dev.unadj))

  ## test1 normal of the unadjusted PB metric
  p.val.2tail.z.unadj <- pnorm(T.chi.unadj, mean=0, sd=1, lower.tail=FALSE)*2
  p.val.1tail.z.unadj <- pnorm(T.chi.unadj, mean=0, sd=1, lower.tail=FALSE)
  cat("\n")
  cat("Test1: T.chi.unadj", "\n")
  cat("          =max(0, (PBmetric-mean(PBmetric.unadj))/ SD(PBmetric.unadj)) statistic", "\n")
  cat("   standard normal approximation test: Mario Roederer:", "\n")
  cat("   ", "unadjusted PB metric (PBmetric.unadj):", chi.sq.unadj, "\n")
  cat("   ", "Statistic used to assess significance of PB metric= max(0, unadjusted PB metric)", "\n")
  cat("                         = (T.chi.unadj):", T.chi.unadj, "\n")
  cat("   ", "one-sided p value (p.val.1tail.z.unadj):", p.val.1tail.z.unadj, "\n")
  cat("   ", "two-sided p value (p.val.2tail.z.unadj):", p.val.2tail.z.unadj, "\n")
  
  
  ## adjusted versions as in the baggerly paper
  chi.sq.adj <- ((2*n.c*n.s)/(n.c + n.s)) * chi.sq.unadj
  
  ## test2 chi-sq, adjusted PB metric
  
  
  p.val.1tail.chi.adj <- pchisq(chi.sq.adj, df=B-1, lower.tail=FALSE)
  
  mean.adj <- B-1
  st.dev.adj <-sqrt( 2*(B-1) ) 
  T.chi.adj <- (chi.sq.adj - mean.adj) / st.dev.adj
  
  
  cat("\n")
  cat("Test2: Adjusted PB metric statistic chi-squared test: Keith A. Baggerly:", "\n")
  cat("   ", "adjusted PB metric (PBmetric.adj):", chi.sq.adj, "\n")
  cat("   ", "degrees of freedom (PB.df):", B-1, "\n")
  cat("   ", "upper tail p value (p.val.1tail.chi.adj):", p.val.1tail.chi.adj, "\n")
  
  
  
  ## test3 normal of the adjusted T.chi.unadj
  p.val.2tail.z.adj <- pnorm(abs(T.chi.adj), mean=0, sd=1, lower.tail=FALSE)*2
  p.val.1tail.z.adj <- pnorm(T.chi.adj, mean=0, sd=1, lower.tail=FALSE)
  cat("\n")
  cat("Test3: Adjusted T.chi.unadj standard normal approximation test: Keith A. Baggerly:", "\n")
  cat("   ", "Adjusted T.chi.unadj (T.chi.adj):", T.chi.adj, "\n")
  cat("   ", "one-sided p value (p.val.1tail.z.adj):", p.val.1tail.z.adj, "\n")
  cat("   ", "two-sided p value (p.val.2tail.z.adj):", p.val.2tail.z.adj, "\n")
  
  ## test4 Pearson's chi-squared test
  cat("\n")
  cat("Test4: Pearson's Chi-Squared Test:", "\n")
  pearson.there <- require(stats)
  if (pearson.there==FALSE) {
    stop("package stats needed but could not be found.")
  }
  pearson.test <- chisq.test(cbind(c.i, s.i))
  print(pearson.test)
  
  pearson.p.val.PBdf<- pchisq(pearson.test$stat, B-1, lower=FALSE)
  
  cat("   ", "upper tail p value when df=", B-1, ": pearson.p.val.PBdf=", pearson.p.val.PBdf, "\n")
  
  
 
  result <- list(PBinType=AnalyType, control.bins=matrix(c.i, ncol=1),
              stim.bins=matrix(s.i, ncol=1),
              total.control=n.c, total.stim=n.s,
              PBmetric.unadj=chi.sq.unadj, T.chi.unadj=T.chi.unadj,
              p.val.2tail.z.unadj= p.val.2tail.z.unadj,
              p.val.1tail.z.unadj= p.val.1tail.z.unadj,
              PBmetric.adj=chi.sq.adj, PB.df=B-1,
              p.val.1tail.chi.adj= p.val.1tail.chi.adj,
              
              T.chi.adj=T.chi.adj, p.val.1tail.z.adj=p.val.1tail.z.adj,
              p.val.2tail.z.adj=p.val.2tail.z.adj,
              pearson.stat=pearson.test$stat, pearson.df=pearson.test$parameter,
              pearson.p.value=pearson.test$p.value,
              pearson.method=pearson.test$method,
              pearson.dataname=pearson.test$data.name,
              pearson.observed=unlist(pearson.test$observed),
              pearson.expected=unlist(pearson.test$expected),
              pearson.p.val.PB.df=pearson.p.val.PBdf)

   class(result) <- "summary.ProbBin.FCS"
  invisible(result)
   
}

  
 
